'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.regexes = exports.boolean = exports.func = exports.number = exports.string = exports.format = exports.oneOf = undefined;

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol ? "symbol" : typeof obj; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _parseFunction = require('../support/parseFunction');

var _parseFunction2 = _interopRequireDefault(_parseFunction);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var oneOf = exports.oneOf = function oneOf(array) {
  return function oneOfInternal(comp) {
    return array.includes(comp);
  };
};
var format = exports.format = function format(regex) {
  return function formatInternal(comp) {
    return regex.test(comp);
  };
};

var _string = function _string(comp) {
  return typeof comp === 'string';
};
exports.string = _string;
var _number = function _number(comp) {
  return typeof comp === 'number';
};
exports.number = _number;
var _func = function _func(comp) {
  return typeof comp === 'function';
};
exports.func = _func;
var _boolean = function _boolean(comp) {
  return typeof comp === 'boolean';
};

exports.boolean = _boolean;

var Shape = function () {
  function Shape(shape) {
    _classCallCheck(this, Shape);

    this.shape = shape;
    this.nonMatchingMessages = [];
    this.previousNonMatchingMessages = [];
  }

  _createClass(Shape, [{
    key: 'lastNonMatches',
    value: function lastNonMatches() {
      return this.previousNonMatchingMessages[this.previousNonMatchingMessages.length - 1];
    }
  }, {
    key: 'matches',
    value: function matches(object) {
      this.recursivelyAccumulateNonMatches(object, this.shape);
      if (this.nonMatchingMessages.length) {
        // Store and reset non-matching messages, for future assertions
        this.previousNonMatchingMessages.push(this.nonMatchingMessages);
        this.nonMatchingMessages = [];
        return false;
      }
      return true;
    }

    /////////////
    // PRIVATE //
    /////////////

  }, {
    key: 'recursivelyAccumulateNonMatches',
    value: function recursivelyAccumulateNonMatches(object, shape) {
      if (shape instanceof Array) {
        this.testArrayMatch(shape, object);
      } else if ((typeof shape === 'undefined' ? 'undefined' : _typeof(shape)) === 'object') {
        this.testObjectMatch(shape, object);
      } else if (typeof shape === 'function') {
        var funcName = (0, _parseFunction2.default)(shape).name.replace('_', '').replace('Internal', '');
        var logAndTestFunction = this[funcName](shape);
        logAndTestFunction(object);
      }

      return true;
    }
  }, {
    key: 'testObjectMatch',
    value: function testObjectMatch(shape, object) {
      var _this = this;

      if ((typeof object === 'undefined' ? 'undefined' : _typeof(object)) === 'object') {
        _lodash2.default.keys(shape).forEach(function (k) {
          return _this.recursivelyAccumulateNonMatches(object[k], shape[k]);
        });
      } else {
        this.nonMatchingMessages.push(JSON.stringify(object) + ' is a ' + (typeof object === 'undefined' ? 'undefined' : _typeof(object)) + ', not an object');
      }
    }
  }, {
    key: 'testArrayMatch',
    value: function testArrayMatch(shape, object) {
      var _this2 = this;

      if (object instanceof Array) {
        object.forEach(function (child) {
          return _this2.recursivelyAccumulateNonMatches(child, shape[0]);
        });
      } else {
        this.nonMatchingMessages.push(JSON.stringify(object) + ' is a ' + (typeof object === 'undefined' ? 'undefined' : _typeof(object)) + ', not an array');
      }
    }
  }, {
    key: 'oneOf',
    value: function oneOf(testFunc) {
      var _this3 = this;

      return function (comp) {
        _this3.logFailure(testFunc, comp, '"' + comp + '" is not within the specified array');
      };
    }
  }, {
    key: 'format',
    value: function format(testFunc) {
      var _this4 = this;

      return function (comp) {
        _this4.logFailure(testFunc, comp, '"' + comp + '" does not match given regex');
      };
    }
  }, {
    key: 'string',
    value: function string() {
      var _this5 = this;

      return function (comp) {
        _this5.logFailure(_string, comp, '"' + comp + '" is a ' + (typeof comp === 'undefined' ? 'undefined' : _typeof(comp)) + ', not a string');
      };
    }
  }, {
    key: 'number',
    value: function number() {
      var _this6 = this;

      return function (comp) {
        _this6.logFailure(_number, comp, '"' + comp + '" is a ' + (typeof comp === 'undefined' ? 'undefined' : _typeof(comp)) + ', not a number');
      };
    }
  }, {
    key: 'func',
    value: function func() {
      var _this7 = this;

      return function (comp) {
        _this7.logFailure(_func, comp, '"' + comp + '" is a ' + (typeof comp === 'undefined' ? 'undefined' : _typeof(comp)) + ', not a function');
      };
    }
  }, {
    key: 'boolean',
    value: function boolean() {
      var _this8 = this;

      return function (comp) {
        _this8.logFailure(_boolean, comp, '"' + comp + '" is a ' + (typeof comp === 'undefined' ? 'undefined' : _typeof(comp)) + ', not a boolean');
      };
    }
  }, {
    key: 'logFailure',
    value: function logFailure(testFunc, comp, errorMsg) {
      if (!testFunc(comp)) {
        this.nonMatchingMessages.push(errorMsg);
      }
    }
  }]);

  return Shape;
}();

var regexes = exports.regexes = {
  iso8601: /\d{4}-\d{2}-\d{2}T\d{2}:\d{2}:\d{2}\.\d{3}[A-Z]{1}/
};

exports.default = Shape;